/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "holdemrender.h"

void holdemRenderInit() {
  // Font
  HOLDEM_GAME_STATE.fontTexture = assetTextureLoad("font.png");
  HOLDEM_GAME_STATE.fontTileset = tilesetCreate(20, 20,
    HOLDEM_GAME_STATE.fontTexture->width,
    HOLDEM_GAME_STATE.fontTexture->height,
    1, 1, 1, 1
  );
  HOLDEM_GAME_STATE.fontBatch = spriteBatchCreate(1024);

  holdemRenderFrameInit();
  holdemRenderSceneInit();
  holdemRenderPlayerInit();
  holdemRenderCardInit();
}

void holdemRender() {
  holdemRenderFrameUpdate();

  // Render things on the left frame buffer
  holdemRenderFrameUseLeft();
  holdemRenderWorld();

  // Render things on the right frame buffer
  holdemRenderFrameUseRight();
  holdemRenderWorld();

  // Finally, render the frame buffers to the back buffer.
  holdemRenderFrameBack();
}

void holdemRenderWorld() {
  uint8_t i, j;
  holdemplayer_t *player;
  uint8_t seat;

  holdemRenderScene();

  // Render the dealer and her hand
  holdemRenderPlayer(HOLDEM_GAME_SEAT_DEALER);
  for(i = 0x00; i < HOLDEM_GAME_STATE.match.cardsFacing; i++) {
    holdemRenderCardForSeat(
      HOLDEM_GAME_SEAT_DEALER,
      HOLDEM_GAME_STATE.match.cards[i],
      HOLDEM_GAME_CARD_SLOT_FLOP0 + i
    );
  }

  // Test
  for(i = 0x00; i < HOLDEM_PLAYER_COUNT; i++) {
    player = HOLDEM_GAME_STATE.match.players + i;
    seat = holdemRenderPlayerGetSeatForPlayer(i);
    holdemRenderPlayer(seat);

    if(player->state & HOLDEM_STATE_FOLDED) continue;

    for(j = 0x00; j < player->cardCount; j++) {
      holdemRenderCardForSeat(seat, player->cards[j], HOLDEM_GAME_CARD_SLOT_HAND0+j);
    }
  }
}
